@@ -24,17 +24,38 @@
 # SOFTWARE.
 
 from typing import Optional, List
+from enum import IntEnum
+
+from PyQt5.QtCore import Qt
+from PyQt5.QtGui import QStandardItemModel, QStandardItem, QFont
+from PyQt5.QtWidgets import QAbstractItemView, QMenu
 
 from electrum.i18n import _
 
-from .util import *
+from .util import MyTreeView, ColorScheme, MONOSPACE_FONT
 
 class UTXOList(MyTreeView):
-    headers = [ _('Address'), _('Label'), _('Amount'), _('Height'), _('Output point')]
-    filter_columns = [0, 1]  # Address, Label
+
+    class Columns(IntEnum):
+        ADDRESS = 0
+        LABEL = 1
+        AMOUNT = 2
+        HEIGHT = 3
+        OUTPOINT = 4
+
+    headers = {
+        Columns.ADDRESS: _('Address'),
+        Columns.LABEL: _('Label'),
+        Columns.AMOUNT: _('Amount'),
+        Columns.HEIGHT: _('Height'),
+        Columns.OUTPOINT: _('Output point'),
+    }
+    filter_columns = [Columns.ADDRESS, Columns.LABEL]
 
     def __init__(self, parent=None):
-        super().__init__(parent, self.create_menu, 1, editable_columns=[])
+        super().__init__(parent, self.create_menu,
+                         stretch_column=self.Columns.LABEL,
+                         editable_columns=[])
         self.setModel(QStandardItemModel(self))
         self.setSelectionMode(QAbstractItemView.ExtendedSelection)
         self.setSortingEnabled(True)
@@ -47,27 +68,32 @@ class UTXOList(MyTreeView):
         self.model().clear()
         self.update_headers(self.__class__.headers)
         for idx, x in enumerate(utxos):
-            address = x.get('address')
-            height = x.get('height')
-            name = x.get('prevout_hash') + ":%d"%x.get('prevout_n')
-            self.utxo_dict[name] = x
-            label = self.wallet.get_label(x.get('prevout_hash'))
-            amount = self.parent.format_amount(x['value'], whitespaces=True)
-            labels = [address, label, amount, '%d'%height, name[0:10] + '...' + name[-2:]]
-            utxo_item = [QStandardItem(x) for x in labels]
-            self.set_editability(utxo_item)
-            utxo_item[0].setFont(QFont(MONOSPACE_FONT))
-            utxo_item[2].setFont(QFont(MONOSPACE_FONT))
-            utxo_item[4].setFont(QFont(MONOSPACE_FONT))
-            utxo_item[0].setData(name, Qt.UserRole)
-            if self.wallet.is_frozen(address):
-                utxo_item[0].setBackground(ColorScheme.BLUE.as_color(True))
-            self.model().insertRow(idx, utxo_item)
+            self.insert_utxo(idx, x)
+
+    def insert_utxo(self, idx, x):
+        address = x.get('address')
+        height = x.get('height')
+        name = x.get('prevout_hash') + ":%d"%x.get('prevout_n')
+        name_short = x.get('prevout_hash')[:10] + '...' + ":%d"%x.get('prevout_n')
+        self.utxo_dict[name] = x
+        label = self.wallet.get_label(x.get('prevout_hash'))
+        amount = self.parent.format_amount(x['value'], whitespaces=True)
+        labels = [address, label, amount, '%d'%height, name_short]
+        utxo_item = [QStandardItem(x) for x in labels]
+        self.set_editability(utxo_item)
+        utxo_item[self.Columns.ADDRESS].setFont(QFont(MONOSPACE_FONT))
+        utxo_item[self.Columns.AMOUNT].setFont(QFont(MONOSPACE_FONT))
+        utxo_item[self.Columns.OUTPOINT].setFont(QFont(MONOSPACE_FONT))
+        utxo_item[self.Columns.ADDRESS].setData(name, Qt.UserRole)
+        utxo_item[self.Columns.OUTPOINT].setToolTip(name)
+        if self.wallet.is_frozen(address):
+            utxo_item[self.Columns.ADDRESS].setBackground(ColorScheme.BLUE.as_color(True))
+        self.model().insertRow(idx, utxo_item)
 
     def selected_column_0_user_roles(self) -> Optional[List[str]]:
         if not self.model():
             return None
-        items = self.selected_in_column(0)
+        items = self.selected_in_column(self.Columns.ADDRESS)
         if not items:
             return None
         return [x.data(Qt.UserRole) for x in items]
