@@ -1,5 +1,8 @@
-import tty, sys
-import curses, datetime, locale
+import tty
+import sys
+import curses
+import datetime
+import locale
 from decimal import Decimal
 import getpass
 
@@ -7,10 +10,12 @@ import electrum
 from electrum.util import format_satoshis, set_verbosity
 from electrum.bitcoin import is_address, COIN, TYPE_ADDRESS
 from electrum.transaction import TxOutput
-from .. import Wallet, WalletStorage
-
-_ = lambda x:x
+from electrum.wallet import Wallet
+from electrum.storage import WalletStorage
+from electrum.network import NetworkParameters, TxBroadcastError, BestEffortRequestFailed
+from electrum.interface import deserialize_server
 
+_ = lambda x:x  # i18n
 
 
 class ElectrumGui:
@@ -27,7 +32,7 @@ class ElectrumGui:
             password = getpass.getpass('Password:', stream=None)
             storage.decrypt(password)
         self.wallet = Wallet(storage)
-        self.wallet.start_threads(self.network)
+        self.wallet.start_network(self.network)
         self.contacts = self.wallet.contacts
 
         locale.setlocale(locale.LC_ALL, '')
@@ -59,7 +64,7 @@ class ElectrumGui:
         self.history = None
 
         if self.network:
-            self.network.register_callback(self.update, ['updated'])
+            self.network.register_callback(self.update, ['wallet_updated', 'network_updated'])
 
         self.tab_names = [_("History"), _("Send"), _("Receive"), _("Addresses"), _("Contacts"), _("Banner")]
         self.num_tabs = len(self.tab_names)
@@ -86,7 +91,7 @@ class ElectrumGui:
         self.set_cursor(0)
         return s
 
-    def update(self, event):
+    def update(self, event, *args):
         self.update_history()
         if self.tab == 0:
             self.print_history()
@@ -179,8 +184,10 @@ class ElectrumGui:
         self.maxpos = 6
 
     def print_banner(self):
-        if self.network:
-            self.print_list( self.network.banner.split('\n'))
+        if self.network and self.network.banner:
+            banner = self.network.banner
+            banner = banner.replace('\r', '')
+            self.print_list(banner.split('\n'))
 
     def print_qr(self, data):
         import qrcode
@@ -195,9 +202,15 @@ class ElectrumGui:
         self.qr.print_ascii(out=s, invert=False)
         msg = s.getvalue()
         lines = msg.split('\n')
-        for i, l in enumerate(lines):
-            l = l.encode("utf-8")
-            self.stdscr.addstr(i+5, 5, l, curses.color_pair(3))
+        try:
+            for i, l in enumerate(lines):
+                l = l.encode("utf-8")
+                self.stdscr.addstr(i+5, 5, l, curses.color_pair(3))
+        except curses.error:
+            m = 'error. screen too small?'
+            m = m.encode(self.encoding)
+            self.stdscr.addstr(5, 1, m, 0)
+
 
     def print_list(self, lst, firstline = None):
         lst = list(lst)
@@ -298,19 +311,22 @@ class ElectrumGui:
     def main(self):
 
         tty.setraw(sys.stdin)
-        while self.tab != -1:
-            self.run_tab(0, self.print_history, self.run_history_tab)
-            self.run_tab(1, self.print_send_tab, self.run_send_tab)
-            self.run_tab(2, self.print_receive, self.run_receive_tab)
-            self.run_tab(3, self.print_addresses, self.run_banner_tab)
-            self.run_tab(4, self.print_contacts, self.run_contacts_tab)
-            self.run_tab(5, self.print_banner, self.run_banner_tab)
-
-        tty.setcbreak(sys.stdin)
-        curses.nocbreak()
-        self.stdscr.keypad(0)
-        curses.echo()
-        curses.endwin()
+        try:
+            while self.tab != -1:
+                self.run_tab(0, self.print_history, self.run_history_tab)
+                self.run_tab(1, self.print_send_tab, self.run_send_tab)
+                self.run_tab(2, self.print_receive, self.run_receive_tab)
+                self.run_tab(3, self.print_addresses, self.run_banner_tab)
+                self.run_tab(4, self.print_contacts, self.run_contacts_tab)
+                self.run_tab(5, self.print_banner, self.run_banner_tab)
+        except curses.error as e:
+            raise Exception("Error with curses. Is your screen too small?") from e
+        finally:
+            tty.setcbreak(sys.stdin)
+            curses.nocbreak()
+            self.stdscr.keypad(0)
+            curses.echo()
+            curses.endwin()
 
 
     def do_clear(self):
@@ -351,15 +367,18 @@ class ElectrumGui:
             self.wallet.labels[tx.txid()] = self.str_description
 
         self.show_message(_("Please wait..."), getchar=False)
-        status, msg = self.network.broadcast_transaction(tx)
-
-        if status:
+        try:
+            self.network.run_from_another_thread(self.network.broadcast_transaction(tx))
+        except TxBroadcastError as e:
+            msg = e.get_message_for_gui()
+            self.show_message(msg)
+        except BestEffortRequestFailed as e:
+            msg = repr(e)
+            self.show_message(msg)
+        else:
             self.show_message(_('Payment sent.'))
             self.do_clear()
             #self.update_contacts_tab()
-        else:
-            self.show_message(_('Error'))
-
 
     def show_message(self, message, getchar = True):
         w = self.w
@@ -376,8 +395,9 @@ class ElectrumGui:
     def network_dialog(self):
         if not self.network:
             return
-        params = self.network.get_parameters()
-        host, port, protocol, proxy_config, auto_connect = params
+        net_params = self.network.get_parameters()
+        host, port, protocol = net_params.host, net_params.port, net_params.protocol
+        proxy_config, auto_connect = net_params.proxy, net_params.auto_connect
         srv = 'auto-connect' if auto_connect else self.network.default_server
         out = self.run_dialog('Network', [
             {'label':'server', 'type':'str', 'value':srv},
@@ -389,13 +409,14 @@ class ElectrumGui:
                 auto_connect = server == 'auto-connect'
                 if not auto_connect:
                     try:
-                        host, port, protocol = server.split(':')
+                        host, port, protocol = deserialize_server(server)
                     except Exception:
                         self.show_message("Error:" + server + "\nIn doubt, type \"auto-connect\"")
                         return False
             if out.get('server') or out.get('proxy'):
                 proxy = electrum.network.deserialize_proxy(out.get('proxy')) if out.get('proxy') else proxy_config
-                self.network.set_parameters(host, port, protocol, proxy, auto_connect)
+                net_params = NetworkParameters(host, port, protocol, proxy, auto_connect)
+                self.network.run_from_another_thread(self.network.set_parameters(net_params))
 
     def settings_dialog(self):
         fee = str(Decimal(self.config.fee_per_kb()) / COIN)
