@@ -35,7 +35,7 @@ import zlib
 from collections import defaultdict
 
 from . import util, bitcoin, ecc
-from .util import PrintError, profiler, InvalidPassword, WalletFileException, bfh
+from .util import PrintError, profiler, InvalidPassword, WalletFileException, bfh, standardize_path
 from .plugin import run_hook, plugin_loaders
 from .keystore import bip44_derivation
 
@@ -73,7 +73,8 @@ class JsonDB(PrintError):
     def __init__(self, path):
         self.db_lock = threading.RLock()
         self.data = {}
-        self.path = os.path.normcase(os.path.abspath(path))
+        self.path = standardize_path(path)
+        self._file_exists = self.path and os.path.exists(self.path)
         self.modified = False
 
     def get(self, key, default=None):
@@ -101,6 +102,20 @@ class JsonDB(PrintError):
                 self.modified = True
                 self.data.pop(key)
 
+    def get_all_data(self) -> dict:
+        with self.db_lock:
+            return copy.deepcopy(self.data)
+
+    def overwrite_all_data(self, data: dict) -> None:
+        try:
+            json.dumps(data, cls=util.MyEncoder)
+        except:
+            self.print_error(f"json error: cannot save {repr(data)}")
+            return
+        with self.db_lock:
+            self.modified = True
+            self.data = copy.deepcopy(data)
+
     @profiler
     def write(self):
         with self.db_lock:
@@ -121,9 +136,12 @@ class JsonDB(PrintError):
             f.flush()
             os.fsync(f.fileno())
 
-        mode = os.stat(self.path).st_mode if os.path.exists(self.path) else stat.S_IREAD | stat.S_IWRITE
+        mode = os.stat(self.path).st_mode if self.file_exists() else stat.S_IREAD | stat.S_IWRITE
+        if not self.file_exists():
+            assert not os.path.exists(self.path)
         os.replace(temp_path, self.path)
         os.chmod(self.path, mode)
+        self._file_exists = True
         self.print_error("saved", self.path)
         self.modified = False
 
@@ -131,14 +149,14 @@ class JsonDB(PrintError):
         return plaintext
 
     def file_exists(self):
-        return self.path and os.path.exists(self.path)
+        return self._file_exists
 
 
 class WalletStorage(JsonDB):
 
     def __init__(self, path, manual_upgrades=False):
         JsonDB.__init__(self, path)
-        self.print_error("wallet path", path)
+        self.print_error("wallet path", self.path)
         self.manual_upgrades = manual_upgrades
         self.pubkey = None
         if self.file_exists():
