@@ -48,7 +48,6 @@ from .util import (NotEnoughFunds, PrintError, UserCancelled, profiler,
                    Fiat, bfh, bh2u, TxMinedInfo)
 from .bitcoin import (COIN, TYPE_ADDRESS, is_address, address_to_script,
                       is_minikey, relayfee, dust_threshold)
-from .version import *
 from .crypto import sha256d
 from .keystore import load_keystore, Hardware_KeyStore
 from .storage import multisig_type, STO_EV_PLAINTEXT, STO_EV_USER_PW, STO_EV_XPUB_PW, WalletStorage
@@ -126,7 +125,7 @@ def sweep_preparations(privkeys, network: 'Network', imax=100):
 
 
 def sweep(privkeys, network: 'Network', config: 'SimpleConfig', recipient, fee=None, imax=100,
-          *, locktime=None):
+          *, locktime=None, tx_version=None):
     inputs, keypairs = sweep_preparations(privkeys, network, imax)
     total = sum(i.get('value') for i in inputs)
     if fee is None:
@@ -142,7 +141,7 @@ def sweep(privkeys, network: 'Network', config: 'SimpleConfig', recipient, fee=N
     if locktime is None:
         locktime = get_locktime_for_new_transaction(network)
 
-    tx = Transaction.from_io(inputs, outputs, locktime=locktime)
+    tx = Transaction.from_io(inputs, outputs, locktime=locktime, version=tx_version)
     tx.set_rbf(True)
     tx.sign(keypairs)
     return tx
@@ -423,7 +422,11 @@ class Abstract_Wallet(AddressSynchronizer):
 
     @profiler
     def get_full_history(self, domain=None, from_timestamp=None, to_timestamp=None,
-                         fx=None, show_addresses=False, show_fees=False):
+                         fx=None, show_addresses=False, show_fees=False,
+                         from_height=None, to_height=None):
+        if (from_timestamp is not None or to_timestamp is not None) \
+                and (from_height is not None or to_height is not None):
+            raise Exception('timestamp and block height based filtering cannot be used together')
         out = []
         income = 0
         expenditures = 0
@@ -438,12 +441,18 @@ class Abstract_Wallet(AddressSynchronizer):
                 continue
             if to_timestamp and (timestamp or now) >= to_timestamp:
                 continue
+            height = tx_mined_status.height
+            if from_height is not None and height < from_height:
+                continue
+            if to_height is not None and height >= to_height:
+                continue
             tx = self.transactions.get(tx_hash)
             item = {
                 'txid': tx_hash,
-                'height': tx_mined_status.height,
+                'height': height,
                 'confirmations': tx_mined_status.conf,
                 'timestamp': timestamp,
+                'incoming': True if value>0 else False,
                 'value': Satoshis(value),
                 'balance': Satoshis(balance),
                 'date': timestamp_to_datetime(timestamp),
@@ -491,6 +500,8 @@ class Abstract_Wallet(AddressSynchronizer):
             summary = {
                 'start_date': start_date,
                 'end_date': end_date,
+                'from_height': from_height,
+                'to_height': to_height,
                 'start_balance': Satoshis(start_balance),
                 'end_balance': Satoshis(end_balance),
                 'income': Satoshis(income),
@@ -523,6 +534,7 @@ class Abstract_Wallet(AddressSynchronizer):
         fiat_rate = self.price_at_timestamp(tx_hash, fx.timestamp_rate)
         fiat_value = fiat_value if fiat_value is not None else self.default_fiat_value(tx_hash, fx, value)
         fiat_fee = tx_fee / Decimal(COIN) * fiat_rate if tx_fee is not None else None
+        item['fiat_rate'] = Fiat(fiat_rate, fx.ccy)
         item['fiat_value'] = Fiat(fiat_value, fx.ccy)
         item['fiat_fee'] = Fiat(fiat_fee, fx.ccy) if fiat_fee else None
         item['fiat_default'] = fiat_default
@@ -719,10 +731,12 @@ class Abstract_Wallet(AddressSynchronizer):
         return tx
 
     def mktx(self, outputs, password, config, fee=None, change_addr=None,
-             domain=None, rbf=False, nonlocal_only=False):
+             domain=None, rbf=False, nonlocal_only=False, *, tx_version=None):
         coins = self.get_spendable_coins(domain, config, nonlocal_only=nonlocal_only)
         tx = self.make_unsigned_transaction(coins, outputs, config, fee, change_addr)
         tx.set_rbf(rbf)
+        if tx_version is not None:
+            tx.version = tx_version
         self.sign_transaction(tx, password)
         return tx
 
