@@ -22,12 +22,16 @@
 # ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
-from . import util
-from .util import profiler, bh2u
-import ecdsa
+
 import hashlib
 import time
 
+import ecdsa
+
+from . import util
+from .util import profiler, bh2u
+
+
 # algo OIDs
 ALGO_RSA_SHA1 = '1.2.840.113549.1.1.5'
 ALGO_RSA_SHA256 = '1.2.840.113549.1.1.11'
@@ -333,8 +337,8 @@ def load_certificates(ca_path):
 
 
 if __name__ == "__main__":
-    import requests
+    import certifi
 
     util.set_verbosity(True)
-    ca_path = requests.certs.where()
+    ca_path = certifi.where()
     ca_list, ca_keyID = load_certificates(ca_path)
